#!/usr/bin/env python
# Copyright 2009 David D Lowe
# released under the GPL v3
"""Script that is run to generate metacity, or gtk preview images
usage:
python theme_preview.py metacity /path/to/themename /path/out.png
or:
GTK2_RC_FILES=/path/to/themename/gtk-2.0/gtkrcfiles python theme_preview.py gtk /path/out.png [/path/to/icondir]
or:
python icons /path/to/themename
or:
GTK2_RC_FILES=/path/to/themename/gtk-2.0/gtkrcfiles python theme_preview.py multiple /path/out.png [metacity /path/to/themename [icons /path/to/themename]
  [wallpaper /path/to/wallpaper.png]
"""

import sys
import gtk
import gobject
import os
import subprocess

def main():
    if len(sys.argv) < 3:
        print >> sys.stderr,  "Invalid command line arguments passed"
        sys.exit(2)
    if sys.argv[1] == "metacity" or sys.argv[1] == "icons":
        themePath = os.path.abspath(os.path.expanduser(sys.argv[2]))
        if not os.path.exists(themePath):
            print >> sys.stderr,  themePath,  "does not exist"
            sys.exit(2)
        if len(sys.argv) < 4:
            print >> sys.stderr,  "Invalid command line arguments passed"
            sys.exit(2)
        outImage = os.path.abspath(os.path.expanduser(sys.argv[3]))
        if sys.argv[1] == "metacity":
            metacity_generate(themePath,  outImage)
        else:
            icon_generate(themePath,  outImage)
    elif sys.argv[1] == "gtk":
        outImage = os.path.abspath(os.path.expanduser(sys.argv[2]))
        iconDir = None
        if len(sys.argv) > 3:
            iconDir = sys.argv[3]
        gtk_generate(outImage,  iconDir)
    elif sys.argv[1] == "multiple":
        outImage = os.path.abspath(os.path.expanduser(sys.argv[2]))
        opts = {}
        ii = 3
        while ii < len(sys.argv):
            if sys.argv[ii] in ("metacity", "icons","wallpaper"):
                opts[sys.argv[ii]] = sys.argv[ii + 1]
            else:
                print >> sys.stderr, "unsupported pigment type: %s" % sys.argv[ii]
                sys.exit(6)
            ii += 2
        skin_generate(outImage, opts)
    else:
        print >> sys.stderr,  "unknown option",  sys.argv[1]
        sys.exit(4)

def metacity_generate(themePath,  outImage):
    window = gtk.Window(gtk.WINDOW_TOPLEVEL)
    window.connect("destroy",  lambda widget: sys.exit(1))
    window.resize(120, 60)
    window.set_property("skip-pager-hint", True)
    window.set_property("skip-taskbar-hint",  True)

    
    import metacity
    theme_preview = metacity.Preview()
    viewport = gtk.Viewport()
    theme_preview.add(viewport)
    window.add(theme_preview)

    subprocess.call(["rm",  "-f",  os.path.expanduser("~/.themes/tmp-pigment-preview")])
    subprocess.call(["ln",  "-s",  themePath,   os.path.expanduser("~/.themes/tmp-pigment-preview")])
    
    gobject.timeout_add(50, lambda: _prepare_metacity(theme_preview))
    gobject.timeout_add(500,  lambda: take_screenshot(theme_preview,  outImage))
    gobject.timeout_add(1000,  lambda: gtk.main_quit())
    
    window.show_all()
    
    gtk.main()
    
    subprocess.call(["rm",  "-r",  os.path.expanduser("~/.themes/tmp-pigment-preview")])
    
    print "Done"
    



def _prepare_metacity(theme_preview, useTemp=True):
    import metacity
    if useTemp:
        theme = metacity.theme_load("tmp-pigment-preview")
    else:
        import gconf
        client = gconf.client_get_default()
        themeName = client.get_string("/apps/metacity/general/theme")
        theme = metacity.theme_load(themeName)
    theme_preview.set_theme(theme)
    theme_preview.set_title("") 

def take_screenshot(widg,  outImage):
    sz = widg.window.get_size()
    pb = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,1,8,sz[0],sz[1])   
    pm = widg.get_snapshot(None)
    pb = pb.get_from_drawable(pm,  pm.get_colormap(),  0, 0, 0, 0,  sz[0],  sz[1])
    if pb is not None:
        pb.save(outImage,  "png")
    else:
        print >> sys.stderr,  "Error getting image"
        sys.exit(3)
            
def gtk_generate(outImage,  iconDir=None):
    window = gtk.Window(gtk.WINDOW_TOPLEVEL)
    window.connect("destroy",  lambda widget: sys.exit(1))
    window.resize(100, 35)
    window.set_property("skip-pager-hint", True)
    window.set_property("skip-taskbar-hint",  True)
    
    hbox = gtk.HBox()
    window.add(hbox)
    button = gtk.Button()
    button.set_label("_Abc")
    hbox.pack_start(button)
    chkbox = gtk.CheckButton()
    hbox.pack_end(chkbox,  False)
    radiobox = gtk.RadioButton()
    hbox.pack_end(radiobox,  False)
    
    window.show_all()

    take_screenshot(hbox,  outImage)
    
    print "Done"
    sys.exit(0)

def icon_generate(themePath,  outImage=None):
    """returns Pixbuf if outImage is None
    if outImage is specified, sys.exit will be called after the image is saved"""
    
    saved = False
    for sizefolder in ("scalable",  "64x64",  "48x48",  "32x32",  "24x24",  "22x22",  "16x16",  "12x12","8x8"):
        if themePath is None: break
        for folder in ("places",  "filesystems"):
            for icon in ("folder.svg",  "folder.png"):
                if os.path.exists(os.path.join(themePath,  sizefolder,  folder,  icon)):
                    pb = gtk.gdk.pixbuf_new_from_file_at_size(os.path.join(themePath,  sizefolder,  folder,  icon),  100,  100)
                    if outImage is None:
                        return pb
                    else:
                        pb.save(outImage,  "png")
                    saved =True
                    break
    if saved == False:
        icontheme = gtk.icon_theme_get_default()
        icc = icontheme.load_icon(gtk.STOCK_MISSING_IMAGE,  gtk.ICON_SIZE_BUTTON,  0)
        if outImage is None:
            return icc
        else:
            icc.save(outImage,  "png")
            print >> sys.stderr, "missing"
            sys.exit(5)
    
    print "Done"
    sys.exit(0)

def skin_generate(outImage, opts):
    # create window to de displayed for half a second
    window = gtk.Window(gtk.WINDOW_TOPLEVEL)
    window.connect("destroy",  lambda widget: sys.exit(1))
    window.resize(120, 60)
    window.set_property("skip-pager-hint", True)
    window.set_property("skip-taskbar-hint",  True)
    
    # prepare metacity preview
    import metacity
    theme_preview = metacity.Preview()
    viewport = gtk.Viewport()
    theme_preview.add(viewport)
    window.add(theme_preview)
    
    # add widgets to it
    hbox = gtk.HBox()
    button = gtk.Button()
    button.set_label("_Abc")
    hbox.pack_start(button)
    chkbox = gtk.CheckButton()
    hbox.pack_start(chkbox,  False)
    radiobox = gtk.RadioButton()
    hbox.pack_start(radiobox,  False)
    image = gtk.Image()
    image.set_from_pixbuf(icon_generate(opts.get("icons",None)).scale_simple(32,32,gtk.gdk.INTERP_BILINEAR))
    hbox.pack_start(image,False)
    
    viewport.add(hbox)  
    
    if opts.has_key("metacity"):
        subprocess.call(["rm",  "-f", os.path.expanduser("~/.themes/tmp-pigment-preview")])
        subprocess.call(["ln",  "-s", opts["metacity"], os.path.expanduser("~/.themes/tmp-pigment-preview")])
        gobject.timeout_add(50, lambda: _prepare_metacity(theme_preview))
    else:
        gobject.timeout_add(50, lambda: _prepare_metacity(theme_preview,False))
        
    gobject.timeout_add(500,  lambda: take_screenshot(theme_preview,  outImage))
    gobject.timeout_add(1000,  lambda: gtk.main_quit())
    
    window.show_all()
    
    gtk.main()
    
    if opts.has_key("metacity"):
        subprocess.call(["rm",  "-r",  os.path.expanduser("~/.themes/tmp-pigment-preview")])
    
    if opts.has_key("wallpaper"):
        wallpaper = gtk.gdk.pixbuf_new_from_file_at_size(opts["wallpaper"],150,113)
        if wallpaper.get_width() < 150 or wallpaper.get_height() < 113:
            # create transparent pixel and scale it
            trans = gtk.gdk.pixbuf_new_from_data('\xff\xff\xff\x00',gtk.gdk.COLORSPACE_RGB,True,8,1,1,4)
            trans = trans.scale_simple(150,113,gtk.gdk.INTERP_NEAREST)
            wallpaper.composite(trans,150 - (wallpaper.get_width() / 2),113 - (wallpaper.get_height() / 2), \
                wallpaper.get_width(),wallpaper.get_height(),150 - (wallpaper.get_width() / 2), \
                113 - (wallpaper.get_height() / 2), 1.0, 1.0, gtk.gdk.INTERP_HYPER,255)
    else:
        # create white pixel and scale it
        wallpaper = gtk.gdk.pixbuf_new_from_data('\xff\xff\xff\xb7', gtk.gdk.COLORSPACE_RGB, False, 8, 1, 1, 4)
        wallpaper = wallpaper.scale_simple(150,113,gtk.gdk.INTERP_NEAREST)
    
    res = gtk.gdk.pixbuf_new_from_file(outImage)
    res.composite(wallpaper,12,25,res.props.width,res.props.height,12,25,1.0,1.0,gtk.gdk.INTERP_HYPER,240)
    
    wallpaper.save(outImage,"png")
    
    print "Done"

    
if __name__ == "__main__":
    main()
